/* gestion des icones couleur par AES avec gestion du cache */
typedef struct  cicon_cache
{
	struct cicon_cache *next;
	int type;
	char name[50];
	int size;
	short width;
	short height;
	CICON *cicon;
} CICON_CACHE;

#ifdef WITH_MYCICONBLK
typedef struct mycicon_blk
{
	ICONBLK		monoblk;			
	CICON 		*mainlist;		
	void 		*params;
} MYCICONBLK;
#endif

static CICON_CACHE *cache_cicon=NULL;

int appl_get_cicon(int type,const char *name,int size,short *cicon_width,short *cicon_height,CICON **cicon_data)
{	AESPB aes_params;
	static INT16 aes_control[AES_CTRLMAX]={131,2,3,1,1};
	INT16 aes_intin[AES_INTINMAX], aes_intout[AES_INTOUTMAX];
	long aes_addrin[AES_ADDRINMAX], aes_addrout[AES_ADDROUTMAX];
	aes_params.control = &aes_control[0];   /* AES Control Array             */
	aes_params.global  = &aes_global[0];    /* AES Global Array              */
	aes_params.intin   = &aes_intin[0];     /* input integer array           */
	aes_params.intout  = &aes_intout[0];    /* output integer array          */
	aes_params.addrin  = &aes_addrin[0];    /* input address array           */
	aes_params.addrout = &aes_addrout[0];   /* output address array          */

	aes_intin[0] = type;
	aes_intin[1] = size;
	aes_addrin[0] = name;
	aes_addrout[0] = (void *)cicon_data;
	aes (&aes_params);

	*cicon_width = aes_intout[1];
	*cicon_height = aes_intout[2];

	return (aes_intout[0]);
}

void free_cicon_cache()
{	CICON_CACHE *cache, *next;
	
	cache = cache_cicon;
	while(cache)
	{
		next = cache->next;
		free(cache);
		cache = next;
	}
	cache_cicon=NULL;
	
	
}

CICON_CACHE *cache_recherche(int type, const char *name, int size)
{	CICON_CACHE *cache;
	cache = cache_cicon;
	while(cache)
	{
		if(cache->type == type && cache->size == size)
		{
			
			if(!strcmp(name,cache->name)) return cache;
		}
		cache = cache->next;
	}
	return NULL;
}

CICON *get_cicon(int type, const char *name, int size, short *cicon_width,short *cicon_height)
{	CICON_CACHE *cache; 
	CICON *cicon_data;
	
	cache = cache_recherche(type, name, size);
	if(cache)
	{
		*cicon_height = cache->height;
		*cicon_width = cache->width;
		return cache->cicon;
	}
	if(appl_get_cicon(type,name,size,cicon_width,cicon_height,&cicon_data))
	{	CICON_CACHE *new_cache; long taille;
		if(type & 0x100) /* icones 32 bits sans masque */
		{
			taille =((long)*cicon_width*(long)*cicon_height*4L);
		}
		else /* icone 24 bits + 1 bit de masque */
		{
			taille =((long)*cicon_width*(long)*cicon_height*(long)cicon_data->num_planes/8L+((long)*cicon_width*(long)*cicon_height/8L));
		}
		if(cicon_data->sel_data) taille = taille + taille;
		 
		new_cache = (CICON_CACHE*) malloc(sizeof(CICON_CACHE)+taille+sizeof(CICON));
		if(new_cache)
		{	
			new_cache->next = cache_cicon;
			cache_cicon = new_cache;
			new_cache->type = type;
			new_cache->size = size;
			new_cache->width = *cicon_width;
			new_cache->height = *cicon_height;
			if(strlen(name)<50) strcpy(new_cache->name,name);
			else new_cache->name[0]=0; /* a revoir cas particulier */
			new_cache->cicon = (CICON *)((unsigned long)new_cache+(unsigned long)sizeof(CICON_CACHE));
			memcpy((void *)new_cache->cicon,cicon_data,taille+sizeof(CICON));
			new_cache->cicon->col_data = (unsigned long) new_cache->cicon->col_data + (unsigned long)new_cache->cicon;
			new_cache->cicon->col_mask = (unsigned long) new_cache->cicon->col_mask + (unsigned long)new_cache->cicon;
			if(new_cache->cicon->sel_data)
			{
				new_cache->cicon->sel_data = (unsigned long) new_cache->cicon->sel_data + (unsigned long)new_cache->cicon;
				new_cache->cicon->sel_mask = (unsigned long) new_cache->cicon->sel_mask + (unsigned long)new_cache->cicon;
			}
			return new_cache->cicon;
		}
	}
	return NULL;
}



/* ajout d'une cicon dans le cache d'une cicon dj en cache */
void add_cicon_cache(int type, char *name, int size, int width, int height, CICON *cicon)
{	CICON_CACHE *new_cache;
	new_cache = (CICON_CACHE*) malloc(sizeof(CICON_CACHE));
	if(new_cache)
	{
		new_cache->type = type;
		new_cache->size = size;
		new_cache->width = width;
		new_cache->height = height;
		if(strlen(name)<50) strcpy(new_cache->name,name);
		else new_cache->name[0]=0; /* a revoir cas particulier */
		new_cache->cicon = cicon;
		new_cache->next = cache_cicon;
		cache_cicon = new_cache;
	}
}


/*
 type : 
 	- 0 : Application
 	- 1 : Extension 
 	- 2 : Fichier
 	- 3 : chemin passe directement
 	- 4 : other
*/
#ifdef WITH_MYCICONBLK
MYCICONBLK *get_ciconblk(int type, const char *name, int size)
#else
CICONBLK *get_ciconblk(int type, const char *name, int size)
#endif
{	CICON *newcicon; 
	short cicon_width,cicon_height;
	
	
	newcicon = get_cicon(type, name, size, &cicon_width, &cicon_height);
	if(newcicon)
	{	
#ifdef WITH_MYCICONBLK
		MYCICONBLK *bloc;
		bloc = malloc(sizeof(MYCICONBLK));
#else	
		CICONBLK *bloc;
		bloc = malloc(sizeof(CICONBLK));
#endif
		if(bloc)
		{	ICONBLK *monobloc;
			monobloc = &bloc->monoblk ;
			bloc->mainlist = newcicon;
#ifdef WITH_MYCICONBLK
			bloc->params = NULL;
#endif
			monobloc->ib_pmask = NULL;
			monobloc->ib_pdata = NULL;
			monobloc->ib_ptext = NULL;
			monobloc->ib_xicon = 0;
			monobloc->ib_yicon = 0;
			monobloc->ib_wicon = cicon_width;
			monobloc->ib_hicon = cicon_height;
			monobloc->ib_char = 0;
			monobloc->ib_xchar = 0;
			monobloc->ib_ychar = 0;
			monobloc->ib_xtext = 0;
			monobloc->ib_ytext = cicon_height;
			monobloc->ib_wtext = 0;
			monobloc->ib_htext = 8;
			
			return bloc;
		}
	}
	return NULL;
}

#ifdef WITH_MYCICONBLK
void del_ciconblk(MYCICONBLK *bloc)
#else
void del_ciconblk(CICONBLK *bloc)
#endif
{
	free((void *)bloc);
}
